/*----------------------------------------------------------------------*/
/* tclanalyzer.c --- The command-line commands to control the analyzer	*/
/*		window.							*/
/*									*/
/*    Written by Tim Edwards January 2005				*/
/*									*/
/*----------------------------------------------------------------------*/

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <tcl.h>
#include <tk.h>

#include "ana.h"
#include "ana_glob.h"
#include "graphics.h"

extern Tcl_Interp *irsiminterp;
extern char **targv;
extern int targc;

extern int lookup();
extern Trptr get_trace();

/*------------------------------------------------------*/
/* The original Analyzer was driven by Xt callbacks, so	*/
/* there is no command syntax for it.  We create one	*/
/* here.						*/
/*------------------------------------------------------*/

/*------------------------------------------------------*/
/* base [trace] [value]					*/
/*------------------------------------------------------*/

int tclirsim_base()
{
    if (targc == 1) {
        /* To-do: return base type */
        return 0;
    }
    if (targv[1][0] ==  'b')
	ChangeBase("binary");
    else if (targv[1][0] ==  'd')
	ChangeBase("decimal");
    else if (targv[1][0] ==  'o')
	ChangeBase("octal");
    else if (targv[1][0] ==  'h')
	ChangeBase("hexidecimal");
    else {
        lprintf(stderr, "Unknown/unhandled numeric base.\n");
	return -1;
    }
    return 0;
}

/*------------------------------------------------------*/
/* cursor [set|move|delta]				*/
/*------------------------------------------------------*/

int tclirsim_cursor()
{
    float rtime;
    TimeType time;	/* TimeType is "long" in ana.h */
    Trptr t;

    if (targc == 1) {
        /* To-do: return time position of cursor */
        return 0;
    }
    if (!strcmp(targv[1], "set")) {
       if (targc != 4) {
	  lprintf(stderr, "Usage: cursor set <trace> <time>.\n");
	  return -1;
       }
       t = get_trace(targv[2]);
       if (sscanf(targv[3], "%f", &rtime) != 1) {
	  lprintf(stderr, "Invalid time value.\n");
	  return -1;
       }
       time = (TimeType)(rtime * 100.0);
       if (t != NULL) SetCursor(t, time);
       MoveCursorToTime(time);
    }

    else if (!strcmp(targv[1], "move")) {
       if (targc == 2) {
	  lprintf(stderr, "Usage: cursor move <time>.\n");
	  return -1;
       }
       else if (sscanf(targv[2], "%f", &rtime) != 1) {
	  lprintf(stderr, "Invalid time value.\n");
	  return -1;
       }
       time = (TimeType)(rtime * 100.0);
       MoveCursorToTime(time);
    }
    else {
        lprintf(stderr, "Unknown cursor option.\n");
	return -1;
    }
    return 0;
}


/*------------------------------------------------------*/
/* simtime <option>					*/
/*------------------------------------------------------*/

int tclirsim_simtime()
{
    int result = 0, idx, x;
    double dt;

    static char *timeOptions[] = {
        "begin", "end", "left", "move", "right", "marker", "cursor", (char *)NULL
    };
    enum timeopt {
        TIME_BEGIN, TIME_END, TIME_LEFT, TIME_MOVE, TIME_RIGHT, TIME_MARKER,
	TIME_CURSOR
    };

    if (targc == 1) {
	lprintf(stderr, "Usage: simtime <option>");
        return -1;
    }

    idx = lookup(targv[1], timeOptions);
    if (idx < 0) return -1;

    switch (idx) {
	case TIME_BEGIN:
            if (targc == 2)
                 Tcl_SetObjResult(irsiminterp, Tcl_NewDoubleObj(analyzer_time_start()));
            break;

        case TIME_END:
            if (targc == 2)
                 Tcl_SetObjResult(irsiminterp, Tcl_NewDoubleObj(analyzer_time_end()));
            break;

        case TIME_LEFT:
            if (targc == 2)
                 Tcl_SetObjResult(irsiminterp, Tcl_NewDoubleObj(analyzer_time_left()));
            break;

        case TIME_RIGHT:
            if (targc == 2)
                 Tcl_SetObjResult(irsiminterp, Tcl_NewDoubleObj(analyzer_time_right()));
            break;

        case TIME_MARKER:
            if (targc == 2) {
                dt = analyzer_time_marker();
                if (dt >= 0.0) {
                    Tcl_SetObjResult(irsiminterp, Tcl_NewDoubleObj(dt));
                }
            }
            break;

        case TIME_CURSOR:
            x = -1;
            if (targc == 3) {
		if (sscanf(targv[2], "%d", &x) != 1)
		     return -1;
            }
            if (x >= 0) {
                dt = (double)analyzer_time_cursor(x);
                Tcl_SetObjResult(irsiminterp, Tcl_NewDoubleObj(dt));
            }
            else {
                Tcl_SetResult(irsiminterp, "Bad position value", 0);
                return TCL_ERROR;
            }
            break;

	case TIME_MOVE:
	    if (targc != 3) {
	       lprintf(stderr, "Requires time at left edge of scope.\n");
	       return -1;
    	    }
	    MoveToT(targv[2]);
	    break;
    }
    return 0;
}

/*------------------------------------------------------*/
/* trace <option>					*/
/*------------------------------------------------------*/

int tclirsim_trace()
{
    Trptr t;
    int result = 0, idx, tidx, y;
    char *tracename, *classname;

    static char *traceOptions[] = {
        "top", "bottom", "order", "base", "class", "cursor", "list",
	"select", (char *)NULL
    };
    enum traceopt {
        TRACE_TOP, TRACE_BOTTOM, TRACE_ORDER, TRACE_BASE, TRACE_CLASS,
        TRACE_CURSOR, TRACE_LIST, TRACE_SELECT
    };

    static char *listOptions[] = {
        "vectors", "nodes", "all", (char *)NULL
    };
    enum listopt {
        LIST_VECTORS, LIST_NODES, LIST_ALL
    };

    if (targc == 1) {
	lprintf(stderr, "Usage: trace <option>");
        return -1;
    }

    idx = lookup(targv[1], traceOptions);
    if (idx < 0) return -1;

    tracename = (targc >= 3) ? targv[2] : NULL;

    switch (idx) {
	case TRACE_TOP:
	    if (targc == 3)
                 Tcl_SetObjResult(irsiminterp,
			Tcl_NewIntObj(analyzer_trace_top(tracename)));
	    break;
        case TRACE_BOTTOM:
            if (targc == 3)
                 Tcl_SetObjResult(irsiminterp,  
                        Tcl_NewIntObj(analyzer_trace_bottom(tracename)));
            break;

        case TRACE_ORDER:
            if (targc == 3)
                 Tcl_SetObjResult(irsiminterp,
                        Tcl_NewIntObj(analyzer_trace_order(tracename)));
            break;

        case TRACE_BASE:
            if (targc == 3)
                 Tcl_SetObjResult(irsiminterp,
                        Tcl_NewIntObj(analyzer_trace_base(tracename)));
            break;

        case TRACE_CLASS:
            if (targc == 3)
                 Tcl_SetResult(irsiminterp, analyzer_trace_class(tracename), NULL);
            break;

        case TRACE_CURSOR:
            y = -1;
            if (targc == 3) {
		if (sscanf(targv[2], "%d", &y) != 1)
		     return -1;
            }
            if (y >= 0) {
                Tcl_SetObjResult(irsiminterp,
                      Tcl_NewStringObj(analyzer_trace_cursor((TimeType)y), -1));
            }
            else {
		lprintf(stderr, "bad position value.\n");
                return -1;
            }
            break;

	case TRACE_LIST:
	    tidx = -1;
	    if (targc >= 3) tidx = lookup(targv[2], listOptions);
	    if (tidx < 0) {
	        lprintf("Usage: trace list [all|nodes|vectors]\n");
	        return -1;
	    }
	    switch(tidx) {
		case LIST_VECTORS:
		    Tcl_SetObjResult(irsiminterp, analyzer_list_vectors(irsiminterp));  
		    break;
		case LIST_NODES:
		    Tcl_SetObjResult(irsiminterp, analyzer_list_nodes(irsiminterp));
		    break;
		case LIST_ALL:
		    Tcl_SetObjResult(irsiminterp, analyzer_list_all(irsiminterp));
		    break;
	    }
	    break;

	case TRACE_SELECT:
	    if (targc == 2) {
	        if (selectedTrace) {
		    SelectTrace(selectedTrace);	/* prints stuff. . . */
	        }
	        else {
		    lprintf(stderr, "must select or specify a trace.\n");
		    return -1;
	        }
	    }
	    else {
	        t = get_trace(targv[2]);
	        if (t == NULL) {
		    lprintf(stderr, "invalid trace name.\n");
		    return -1;
	        }
	        SelectTrace(t);
	    }
	    break;
    }
    return result;
}

/*------------------------------------------------------*/
/* zoom [in|out]					*/
/*------------------------------------------------------*/

int tclirsim_zoom()
{
    if (targc == 1) {
        /* To-do: return zoom factor */
        return 0;
    }

    if (targv[1][0] ==  'i')
	Zoom("in");
    else if (targv[1][0] ==  'o')
	Zoom("out");
    else {
        lprintf(stderr, "Unknown zoom option.  Try 'in' or 'out'.\n");
	return -1;
    }
    return 0;
}

